/***************************************************************************
*   Copyright (C) 2008 by Jonathan Schmidt-Dominé                         *
*   the-user@gradespro.com                                                *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
***************************************************************************/

#ifndef DYNH_PTR_CPP

#define DYNH_PTR_CPP

#include "dynh_ptr.h"
#include <stdexcept>

using namespace std;

template<typename type> PTR_ACCESS_DEF type& dynh_ptr<type>::operator*() const
{
    if(is_real)
    {
        if(rptr) return *rptr;
        throw domain_error("Tried to call dynh_ptr::operator*(), but the pointer is NULL");
    }
    return **iptr;
}

template<typename type> PTR_ACCESS_DEF type* dynh_ptr<type>::operator->() const
{
    if(is_real)
    {
        if(rptr) return rptr;
        throw domain_error("Tried to call dynh_ptr::operator->(), but the pointer is NULL");
    }
    return iptr->operator->();
}

template<typename type> PTR_ACCESS_DEF dynh_ptr<type>& dynh_ptr<type>::next() const
{
    if(is_real)
        throw domain_error("Tried to call dynh_ptr::next(), but there is no next-object");
    return *iptr;
}

template<typename type> PTR_ACCESS_DEF dynh_ptr<type>& dynh_ptr<type>::operator=(type& ref)
{
    rptr = &ref;
    is_real = true;
    return *this;
}

template<typename type> PTR_ACCESS_DEF dynh_ptr<type>& dynh_ptr<type>::operator=(ptr_interface<type>& ref)
{
    iptr = &ref;
    is_real = false;
    return *this;
}

template<typename type> PTR_ACCESS_DEF bool dynh_ptr<type>::operator==(ptr_interface<type>& optr) const
{
    return &(**this) == &(**optr);
}

template<typename type> PTR_ACCESS_DEF bool dynh_ptr<type>::points_at(dynh_ptr<type>& optr) const
{
    return !is_real && (iptr == &optr || iptr->points_at(optr));
}

template<typename type> PTR_ACCESS_DEF bool dynh_ptr<type>::points_at(type& optr) const
{
    if(is_real)
        return rptr == &optr;
    return iptr->points_at(optr);
}

template<typename type> PTR_ACCESS_DEF void dynh_ptr<type>::unbind()
{
    rptr = 0;
    is_real = true;
}

template<typename type> PTR_ACCESS_DEF type*& dynh_ptr<type>::get_ptr()
{
    return is_real ? rptr : iptr->get_ptr();
}

/**************************************************************************************/

template<typename type> PTR_ACCESS_DEF controlled_dynh_ptr<type>& controlled_dynh_ptr<type>::operator=(dynh_ptr<type>& ref)
{
    cout << "hi" << endl;
    if(ref.points_at(*this))
        throw domain_error("Tried to bind a controlled_dynh_ptr to a dynh_ptr, but the dynh_ptr is bound to the controlled_dynh_ptr");
    dynh_ptr<type>::operator=(ref);
    return *this;
}

#endif
