/***************************************************************************
*   Copyright (C) 2008 by Jonathan Schmidt-Dominé                         *
*   the-user@gradespro.com                                                *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
***************************************************************************/

/*
Diese Zeiger-ähnliche Klasse versucht nicht, das Speichermanagement zu verbessern.
Vielmehr steht hier eine andere Rolle von Zeigern im Mittelpunkt:
Zeiger, die sich auf ein anderes Objekt beziehen und dabei nicht die Kontrolle
über den Speicher übernehmen. Hierbei treten auch schonmal Typen wie int**
oder foo*** auf, wenn ein Zeiger sich immer auf das Objekt beziehen soll,
auf das sich ein anderer bezieht, es existiert also eine Hierarchie, int***
bezieht sich auch int**, dieses auf int* und dieses auf int.
Diese Klasse ermöglicht es, diese Hierarchie dynamisch zu machen, der Typ
bindet nicht mehr an eine bestimmte Stelle in der Hierarchie.
Daher der Name: dyn(amisch)h(ierarchischer)_ptr(pointer).
Beispiel:
int x = 4711, y = 1337;
dynh_ptr<int> p1 (x);
dynh_ptr<int> p2 (p1); // p2 bezieht sich auf p1
dynh_ptr<int> p3;      // p3 ist ungebunden
p3 = p2;               // zeigt auf p2, *p1 == *p2 == *p3 == 4711
p3 = y;                // p3 zeigt nun direkt auf y
p2 = p3;               // nun steht p2 in der Hierarchie unter p3
p3 = p2;               // Zirkelbeziehung
--------------------------------------
Die Klasse ist eben dafür gedacht, dass eine Zirkelbeziehung nicht vorkommen kann,
es wird keine Prüfung ausgeführt, dafür hat man jedoch keine unnötigen Operationen.
(C) by Jonathan Schmidt-Dominé
*/


#ifndef DYNH_PTR_H

#define DYNH_PTR_H

#include "PTR_ACCESS.h"
#include "ptr_interface.h"
#include <cstddef>
#include <stdexcept>
using namespace std;

using namespace std;

template<typename type> class dynh_ptr
: public ptr_interface<type>
{
    private:
        union                 // Entweder Verweis auf anderen Zeiger oder auf type
        {
            type* rptr;
            ptr_interface<type>* iptr;
        };
        bool is_real;
        PTR_ACCESS type*& get_ptr();
    public:
        dynh_ptr() : rptr(0), is_real(1)
        {}
        dynh_ptr(type & ref) : rptr(&ref), is_real(1)
        {}
        dynh_ptr(dynh_ptr<type> & ref) : iptr(&ref), is_real(0)
        {}
        PTR_ACCESS dynh_ptr<type>& operator=(type& ref);
        PTR_ACCESS dynh_ptr<type>& operator=(ptr_interface<type>& ref);
        PTR_ACCESS type& operator*() const;
        PTR_ACCESS type* operator->() const;
        PTR_ACCESS bool operator==(ptr_interface<type>& optr) const;
        PTR_ACCESS dynh_ptr<type>& next() const;
        PTR_ACCESS bool points_at(dynh_ptr<type>& optr) const;
        PTR_ACCESS bool points_at(type& optr) const;
        PTR_ACCESS void unbind();
};

template<typename type> class controlled_dynh_ptr : public dynh_ptr<type>
{
    public:
        PTR_ACCESS controlled_dynh_ptr<type>& operator=(dynh_ptr<type>& ref);
};

#ifndef V_PTR
    #include "jpointer.cpp"
#endif

#endif

