/***************************************************************************
*   Copyright (C) 2008 by Jonathan Schmidt-Dominé                         *
*   the-user@gradespro.com                                                *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
***************************************************************************/

#ifndef JPOINTER_CPP

#include <stdexcept>

using namespace std;

#define JPOINTER_CPP

#include "jpointer.h"

template<typename type> jpointer<type>& jpointer<type>::operator=(type* p)
{
    if(ptr)
    {
        if(*count == 1)
        {
            delete ptr;
            ptr = 0;
            delete count;
            count = 0;
        }
        else
            --(*count);
    }
    ptr = p;
    count = new size_t(1);
    return *this;
}
template<typename type> jpointer<type>& jpointer<type>::operator=(const jpointer<type>& jp)
{
    if(jp.ptr == 0)
    {
        ptr = 0;
        count = 0;
        return *this;
    }
    if(ptr = jp.ptr)
        return *this;
    if(*count == 1)
    {
        delete ptr;
        ptr = 0;
        delete count;
        count = 0;
    }
    else
        --(*count);
    ptr = jp.ptr;
    count = jp.count;
    ++(*count);
    return *this;
}
template<typename type> PTR_ACCESS_DEF type& jpointer<type>::operator*() const
{
    if(ptr)
        return *ptr;
    throw domain_error("Tried to call jpointer::operator*(), but the pointer is NULL");
}
template<typename type> PTR_ACCESS_DEF type* jpointer<type>::operator->() const
{
    if(ptr)
        return ptr;
    throw domain_error("Tried to call jpointer::operator->(), but the pointer is NULL");
}
template<typename type> PTR_ACCESS_DEF bool jpointer<type>::points_at(type& optr) const
{
    return ptr == &optr;
}
template<typename type> PTR_ACCESS_DEF bool jpointer<type>::points_at(ptr_interface<type>& optr) const
{
    if(optr.get_ptr() == ptr)
        return true;
    return false;
}
template<typename type> PTR_ACCESS_DEF void jpointer<type>::destroy()
{
    if(count)
    {
        if(*count == 1)
        {
            delete ptr;
            delete count;
        }
        else
            --(*count);
    }
}
template<typename type> PTR_ACCESS_DEF void jpointer<type>::unbind()
{
    destroy();
    count = 0;
    ptr = 0;
}
template<typename type> PTR_ACCESS_DEF type*& jpointer<type>::get_ptr()
{
    return ptr;
}
template<typename type> jpointer<type>::~jpointer()
{
    destroy();
}

#endif
