/***************************************************************************
*   Copyright (C) 2008 by Jonathan Schmidt-Dominé                         *
*   the-user@gradespro.com                                                *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
***************************************************************************/

/*
Ein Java-ähnlicher Zeiger, der einen Verwendungszähler benutzt, um nicht genutzte Objekte zu löschen.
Fehler, die durch die Dereferenzierung von NULL-Pointern entstehen, können abgefangen werden
und führen nicht zu einem Speicherzugriffsfehler.
Die Klasse ist Ausnahmensicher, da der Destruktor alle nicht mehr benötigten Daten löschen wird.
*/

#ifndef JPOINER_H

#define JPOINER_H

#include "PTR_ACCESS.h"
#include "ptr_interface.h"
#include <cstddef>
#include <stdexcept>
using namespace std;

template<typename type> class jpointer : public ptr_interface<type>
{
    private:
        type* ptr;
        std::size_t* count;
        PTR_ACCESS void destroy();
        PTR_ACCESS type*& get_ptr();
    public:
        jpointer() : ptr(0), count(0)
        {
            get_ptr();
        }
        jpointer(type* p) : ptr(p), count(new size_t(1))
        {}
        jpointer(const jpointer<type>& jp) : ptr(jp.ptr), count(jp.count)
        {
            if(jp.ptr != 0)
                ++(*count);
        }
        jpointer<type>& operator=(type*);
        jpointer<type>& operator=(const jpointer<type>&);
        PTR_ACCESS type& operator*() const;
        PTR_ACCESS type* operator->() const;
        PTR_ACCESS bool points_at(type& optr) const;
        PTR_ACCESS bool points_at(ptr_interface<type>&) const;
        PTR_ACCESS void unbind();
        virtual ~jpointer();
};

#ifndef V_PTR
    #include "jpointer.cpp"
#endif

#endif
