/***************************************************************************
*   Copyright (C) 2008 by Jonathan Schmidt-Dominé                         *
*   the-user@gradespro.com                                                *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
***************************************************************************/

#ifndef SLAVE_PTR_CPP
#define SLAVE_PTR_CPP

#include "slave_ptr.h"

template<typename type> slave_ptr<type>::slave_ptr() : ptr(new type*)
{
	*ptr = 0;
}

template<typename type> slave_ptr<type>::slave_ptr(type* nptr) : ptr(new type*)
{
    *ptr = nptr;
}

template<typename type> slave_ptr<type>::slave_ptr(ptr_interface<type>& nptr) : ptr(&nptr.get_ptr())
{
}

template<typename type> PTR_ACCESS_DEF type& slave_ptr<type>::operator*() const
{
    if(ptr && *ptr)
        return **ptr;
    throw domain_error("Tried to call slave_ptr::operator*(), but the pointer is NULL");
}

template<typename type> PTR_ACCESS_DEF type* slave_ptr<type>::operator->() const
{
    if(ptr && *ptr)
        return *ptr;
    throw domain_error("Tried to call slave_ptr::operator->(), but the pointer is NULL");
}

template<typename type> PTR_ACCESS_DEF void slave_ptr<type>::unbind()
{
    ptr = 0;
}

template<typename type> PTR_ACCESS_DEF bool slave_ptr<type>::points_at(type& optr) const
{
    return *ptr == &optr;
}

template<typename type> PTR_ACCESS_DEF bool slave_ptr<type>::points_at(ptr_interface<type>& optr) const
{
    return *ptr == optr.get_ptr();
}

template<typename type> slave_ptr<type>::~slave_ptr()
{
    unbind();
}

template<typename type> PTR_ACCESS_DEF slave_ptr<type>& slave_ptr<type>::operator=(ptr_interface<type>& nptr)
{
    ptr = &nptr.get_ptr();
    return *this;
}

template<typename type> PTR_ACCESS_DEF slave_ptr<type>& slave_ptr<type>::operator=(type*& nptr)
{
    ptr = nptr;
    return *this;
}

template<typename type> PTR_ACCESS_DEF type*& slave_ptr<type>::get_ptr()
{
    return *ptr;
}

#endif
